-- Copyright (C) 2008 Papavasileiou Dimitris                             
--                                                                      
-- This program is free software: you can redistribute it and/or modify 
-- it under the terms of the GNU General Public License as published by 
-- the Free Software Foundation, either version 3 of the License, or    
-- (at your option) any later version.                                  
--                                                                      
-- This program is distributed in the hope that it will be useful,      
-- but WITHOUT ANY WARRANTY; without even the implied warranty of       
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
-- GNU General Public License for more details.                         
--                                                                      
-- You should have received a copy of the GNU General Public License    
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

require "network"
require "resources"
require "derived"

function callhooks (hooks, ...)
   if type (hooks) == "table" then
      for _, hook in pairs(hooks) do
	 if ... and type(...) == "table" then
	    hook (unpack(...))
	 else
	    hook (...)
	 end
      end
   elseif type (hooks) == "function" then
      if ... and type(...) == "table" then
	 hooks(unpack(...))
      else
	 hooks (...)
      end
   end
end

graphics.close = function ()
		    common.iterate = false
		 end

-- Right now full shading only works on
-- 'certain' GPUs.  The rest will have
-- to do with fixed-pipeline toon shading.

if not options.notoon and
   (not string.find (graphics.vendor, "NVIDIA") or
    not string.find (graphics.extensions, "fragment_shader")) then
   options.toon = true
end

-- These values are fixed.

graphics.title = "Billiards"
graphics.hide = false
graphics.perspective = {-1, 1, -1, 1, 0.01, 20}
graphics.canvas = options.toon and {0.7, 0.7, 0.6} or {0.07, 0.07, 0.075}

dynamics.collision = {}

billiards = {}
billiards.waiting = {}
billiards.looking = {}
billiards.aiming = {}
billiards.ballcollision = {}
billiards.cushioncollision = {}
billiards.cuecollision = {}
billiards.striking = {}
billiards.newshot = {}
billiards.turning = {}
billiards.zooming = {}
billiards.panning = {}
billiards.adjusting = {}
billiards.grabbing = {}
billiards.releasing = {}
billiards.opening = {}
billiards.history = {}

bindings = {}

-- Load the default configuration.

resources.dofile "scripts/defaults.lua"

-- Load per-user configuration.

print "Reading local user configuration."

local rcscript = loadfile (os.getenv ("HOME") .. "/.billiards")

if rcscript then
   print ("  " .. os.getenv ("HOME") .. "/.billiards")
   rcscript ()
else
   local defaultsfile, rcfile, error

   print "Creating user configuration with default values."

   -- If none exists create one by copying the faults.

   defaultsfile = io.open ("scripts/defaults.lua", "r")
   rcfile, error = io.open (os.getenv ("HOME") .. "/.billiards", "w")

   if defaultsfile and rcfile then
      local defaults

      defaults = defaultsfile:read "*a"
      rcfile:write (defaults)
   else
      print (error)
   end

   io.close (defaultsfile)
   io.close (rcfile)
end

resources.dofile "scripts/collision.lua"
resources.dofile "scripts/observer.lua"
resources.dofile "scripts/room.lua"
resources.dofile "scripts/cue.lua"
resources.dofile "scripts/rack.lua"
resources.dofile "scripts/game.lua"
resources.dofile "scripts/display.lua"
resources.dofile "scripts/httpd.lua"
-- resources.dofile "scripts/sounds.lua"

-- billiards.striking.slow = function()
-- 				dynamics.timescale = 0.25
-- 			     end

-- common.iterate = false
