-- Copyright (C) 2008 Papavasileiou Dimitris                             
--                                                                      
-- This program is free software: you can redistribute it and/or modify 
-- it under the terms of the GNU General Public License as published by 
-- the Free Software Foundation, either version 3 of the License, or    
-- (at your option) any later version.                                  
--                                                                      
-- This program is distributed in the hope that it will be useful,      
-- but WITHOUT ANY WARRANTY; without even the implied warranty of       
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
-- GNU General Public License for more details.                         
--                                                                      
-- You should have received a copy of the GNU General Public License    
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

require "transforms"
require "resources"
require "frames"
require (options.toon and "toon" or "shading")

local w = billiards.tablewidth
local h = billiards.tableheight
local h_0 = billiards.cushionheight

graph.observer = bodies.observer

-- The lighting setup.

graph.lamp = options.toon and toon.lamp {
   position = {0, 0, 0.8 * w},
   orientation = transforms.euler (180, 0, 0),
   
   intensity = {[0] = 0.1, [0.04] = 0.4, [0.15] = 0.8},
   ambience = 0.4
} or shading.light {
   position = {0, 0, 0.8 * w},
   orientation = transforms.euler (180, 0, 0),

   intensity = resources.clamped "scripts/imagery/light/spot.lc",
   attenuation = {1.1, 0, 0},
   
   offset = {1, 2},
   size = {2048, 1024},
   volume = {-0.25 * w - 0.125,
	     0.25 * w + 0.125,
		-0.25 * h - 0.125,
	     0.25 * h + 0.125,
	     0.35 * w,
	     1.4 * w}
}

graph.ambience = not options.toon and shading.ambient {
   intensity = resources.clamped "scripts/imagery/light/roomambience.lc"
}

graph.fog = not options.toon and shading.fog {
   quadratic = 0.04,
   offset = 3,
   color = graphics.canvas
}

graph.room = bodies.environment {
   -- The floor.

   floor = not options.toon and bodies.plane {
      isfloor = true,

      position = {0, 0, -0.75},

      surface = shading.phong {
	 diffuse = resources.periodic "scripts/imagery/diffuse/parquet.lc",
	 specular = resources.periodic "scripts/imagery/specular/parquet.lc",
	 parameter = 64,
	 
	 mesh = resources.static "scripts/meshes/floor.lc"
      }
   }
}

graph.pooltable = resources.dofile "scripts/pooltable.lua"
graph.caromtable = resources.dofile "scripts/caromtable.lua"
